"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isAUID = exports.generateAUID_dom_toRaw = exports.generateAUID_dom_fromRaw = exports.generateAUID_dom_raw = exports.generateAUID_dom = exports.generateAUID_toRaw = exports.generateAUID_fromRaw = exports.generateAUID_raw = exports.generateAUID = void 0;
const crypto_1 = require("crypto");
/**
 * Generates a crytografically strong AUID
 *
 * With `(2 ^ 4) ^ (16*4)` or `2 ^ 256` or `1.1579209e+77` possible combinations
 */
function generateAUID() {
    const bytes = (0, crypto_1.randomBytes)(32).toString('hex'); // 64 hex characters
    return `${bytes.slice(0, 16)}-${bytes.slice(16, 32)}-${bytes.slice(32, 48)}-${bytes.slice(48, 64)}`;
}
exports.generateAUID = generateAUID;
;
/**
 * Generate a random raw Buffer data that represents an AUID
 */
function generateAUID_raw() {
    return (0, crypto_1.randomBytes)(32);
}
exports.generateAUID_raw = generateAUID_raw;
;
/**
 * Generate an AUID from the raw Buffer data
 */
function generateAUID_fromRaw(data) {
    const bytes = data.toString("hex");
    return `${bytes.slice(0, 16)}-${bytes.slice(16, 32)}-${bytes.slice(32, 48)}-${bytes.slice(48, 64)}`;
}
exports.generateAUID_fromRaw = generateAUID_fromRaw;
;
/**
 * Generate a raw Buffer data from an AUID
 */
function generateAUID_toRaw(data) {
    const hexString = data.replace(/-/g, "");
    return Buffer.from(hexString, "hex");
}
exports.generateAUID_toRaw = generateAUID_toRaw;
;
/**
 * Generates a crytografically strong AUID **inside of the browser environment**
 *
 * With `(2 ^ 4) ^ (16*4)` or `2 ^ 256` or `1.1579209e+77` possible combinations
 */
function generateAUID_dom() {
    return Array.from(crypto.getRandomValues(new Uint8Array(32)))
        .map(byte => byte.toString(16).padStart(2, "0"))
        .join("")
        .match(/.{1,16}/g)
        .join("-");
}
exports.generateAUID_dom = generateAUID_dom;
;
function generateAUID_dom_raw() {
    const bytes = new Uint8Array(32);
    crypto.getRandomValues(bytes);
    return bytes;
}
exports.generateAUID_dom_raw = generateAUID_dom_raw;
;
function generateAUID_dom_fromRaw(data) {
    // Convert the bytes to a hexadecimal string
    const hexArray = Array.from(data).map(byte => byte.toString(16).padStart(2, "0")).join("");
    return `${hexArray.slice(0, 16)}-${hexArray.slice(16, 32)}-${hexArray.slice(32, 48)}-${hexArray.slice(48, 64)}`;
}
exports.generateAUID_dom_fromRaw = generateAUID_dom_fromRaw;
;
/**
 * Generate a raw Uint8Array from an AUID
 */
function generateAUID_dom_toRaw(data) {
    // Remove the hyphens from the AUID string
    const hexString = data.replace(/-/g, "");
    // Create a Uint8Array with half the length of the hex string
    const byteArray = new Uint8Array(hexString.length / 2);
    // Populate the Uint8Array by parsing the hex string in pairs
    for (let i = 0; i < byteArray.length; i++) {
        byteArray[i] = (parseInt(hexString[i * 2], 16) << 4) | parseInt(hexString[i * 2 + 1], 16);
    }
    return byteArray;
}
exports.generateAUID_dom_toRaw = generateAUID_dom_toRaw;
;
/**
 * Fullproof regex for AUIDs
 */
exports.isAUID = new RegExp(/^[a-f0-9]{16}-[a-f0-9]{16}-[a-f0-9]{16}-[a-f0-9]{16}$/);
