import { log } from "console";
import { randomBytes } from "crypto";

/**
 * Template string type for AUIDs
 */
export type AUID = `${string}-${string}-${string}-${string}`;

/**
 * Generates a crytografically strong AUID
 * 
 * With `(2 ^ 4) ^ (16*4)` or `2 ^ 256` or `1.1579209e+77` possible combinations
 */
export function generateAUID(): AUID {
    const bytes = randomBytes(32).toString('hex'); // 64 hex characters
    return `${bytes.slice(0, 16)}-${bytes.slice(16, 32)}-${bytes.slice(32, 48)}-${bytes.slice(48, 64)}` as AUID;
};

/**
 * Generate a random raw Buffer data that represents an AUID
 */
export function generateAUID_raw(): Buffer {
	return randomBytes(32);
};

/**
 * Generate an AUID from the raw Buffer data
 */
export function generateAUID_fromRaw(data: Buffer): AUID {
	const bytes = data.toString("hex");
    return `${bytes.slice(0, 16)}-${bytes.slice(16, 32)}-${bytes.slice(32, 48)}-${bytes.slice(48, 64)}` as AUID;
};

/**
 * Generate a raw Buffer data from an AUID
 */
export function generateAUID_toRaw(data: AUID): Buffer {
	const hexString = data.replace(/-/g, "");
    return Buffer.from(hexString, "hex");
};

/**
 * Generates a crytografically strong AUID **inside of the browser environment**
 * 
 * With `(2 ^ 4) ^ (16*4)` or `2 ^ 256` or `1.1579209e+77` possible combinations
 */
export function generateAUID_dom(): AUID {
    return Array.from(crypto.getRandomValues(new Uint8Array(32)))
        .map(byte => byte.toString(16).padStart(2, "0"))
        .join("")
        .match(/.{1,16}/g)!
        .join("-") as AUID;
};

export function generateAUID_dom_raw(): Uint8Array {
	const bytes = new Uint8Array(32);
	crypto.getRandomValues(bytes);
	return bytes;
};

export function generateAUID_dom_fromRaw(data: Uint8Array): AUID {
	// Convert the bytes to a hexadecimal string
	const hexArray = Array.from(data).map(byte => byte.toString(16).padStart(2, "0")).join("");
	return `${hexArray.slice(0, 16)}-${hexArray.slice(16, 32)}-${hexArray.slice(32, 48)}-${hexArray.slice(48, 64)}` as AUID;
};

/**
 * Generate a raw Uint8Array from an AUID
 */
export function generateAUID_dom_toRaw(data: AUID): Uint8Array {
    // Remove the hyphens from the AUID string
    const hexString = data.replace(/-/g, "");

    // Create a Uint8Array with half the length of the hex string
    const byteArray = new Uint8Array(hexString.length / 2);

    // Populate the Uint8Array by parsing the hex string in pairs
    for (let i = 0; i < byteArray.length; i++) {
        byteArray[i] = (parseInt(hexString[i * 2], 16) << 4) | parseInt(hexString[i * 2 + 1], 16);
    }

    return byteArray;
};

/**
 * Fullproof regex for AUIDs
 */
export const isAUID = new RegExp(/^[a-f0-9]{16}-[a-f0-9]{16}-[a-f0-9]{16}-[a-f0-9]{16}$/);
